**********************************************************************
**********************************************************************
** Name:	SOKOBAN
**
** Category:	GAME
**
** Author:	Daniel Lidstrm
**		mail:	danli97@ite.mh.se
**		web:	www.ite.mh.se/~danli97/
**		Home:
**			Nackstavgen 20D
**			853 52  Sundsvall
**			Sweden
**
** Abstract:	The game of Sokoban.
**
**
**  Date	Prog		Modification
** ------	----	----------------------------
** 010305	DL	Implemented and commented
** 010308	DL	Font and store-key working
** 010312	DL	V1.0 Offkey and timeout
** 010312	DL	V1.1 Clean up. Better STO code. Added Rcl key.
**			Added protection against corrupt SOpar.
** 010322	DL	V1.2 Now also for the HP49G. Added keys.
**====================================================================

ASSEMBLE
SOKOSTART
RPL
::
 INCLUDE LEV.S
 ' ID SOpar
 Sys@
 ITE
 ::		( we have a stored object )
  TOTEMPOB	( dereference object )
CODE
	GOSBVL	=SAVPTR
	A=DAT1	A
	D1=A
	A=DAT1	A
	LC(5)	=DOEXT0
	?C=A	A
	GOYES	typeOk
	LC(5)	=SETTYPEERR
	GOTO	GetptrEvalC
typeOk	LC(5)	=DOLIST
	DAT1=C	A
	D1=D1+	5
	LC(5)	=DOBINT
	DAT1=C	A
	GOVLNG	=GETPTRLOOP
ENDCODE
* { #Level LibData }
  INCOMPDROP
ASSEMBLE
	CON(5)	=DOBINT
	CON(5)	(SAVED)+10
RPL
  OVER OSIZE
  #<>case
  ::
   2DROP
   $ "Bad SOpar!"
   DO$EXIT
  ;
ASSEMBLE
	CON(5)	=DOBINT
	CON(5)	34*64
RPL
  EXPAND
  SWAPTRUE
 ;
 ::	( we do not have a stored object )
ASSEMBLE
	CON(5)	=DOEXT0	* an empty libdat
	CON(5)	5
RPL
ASSEMBLE
	CON(5)	=DOBINT
	CON(5)	TOTAL
RPL
  EXPAND
  FALSE
 ;
 RECLAIMDISP			( prepare display )
 TURNMENUOFF
ASSEMBLE
		CON(5)	=DOCODE
		REL(5)	->end
* Buffer on stack:
		ABASE	0
LEV		ALLOC	2
LEV.RAM		ALLOC	LEVSZ
SAVED		ALLOC	0
OFFSCR		ALLOC	34*64
TOTAL		ALLOC	0

DELAY		EQU	10
sWAIT		EQU	0
sSAVE		EQU	1
**********************************************************************
* Register usage
* R0.A=LEVELS
* R1.A=Offscreen buffer
* R2.A=CUR LEV(ROM)
* R3.A=Buffer
**********************************************************************
* start of program
* we have on stack:
* Levels LibBuffer #Level TRUE / Levels LibBuffer FALSE
**********************************************************************
start		GOSUBL	InitVars	get stuff from stack
		GOSBVL	=DisableIntr
restartMain	GOSUBL	InitLev		this checks sSAVE, and acts accordingly
		GOSUBL	DrawLev#	draw level number
*********************************
* MAIN LOOP
*********************************
		GOSUBL	DRAWLEV
		GOSUB	SWAPBUF
MAINLP
		INCLUDE	time.s		48/49 specific timeout code

chkKeys		GOSUB	KeyNum		B.B = keynum
		?B=0	B
		GOYES	MAINLP
		GOSUB	passKeys
		
		INCLUDE	keytable.s
**********************************************************************
* Key subroutines
**********************************************************************
doBadKey	GOSUB	WAITNOKEY
		GOTO	commonKey

doLeft		GOSUB	GO_LEFT
		GOTO	commonKey

doRight		GOSUB	GO_RIGHT
		GOTO	commonKey

doUp		GOSUB	GO_UP
		GOTO	commonKey

doDown		GOSUB	GO_DOWN
		GOTO	commonKey

doNeg		GOSUB	TOGDelay
		GOSUB	WAITNOKEY
		GOTO	commonKey

doEnter
doDrop		GOSUB	WAITNOKEY
		GOTO	doExit
		GOTO	commonKey

doPlus		GOSUB	LEV+
		ST=0	sSAVE
		GOTO	restartMain

doMinus		GOSUB	LEV-
		ST=0	sSAVE
		GOTO	restartMain

doSto		GOTO	doStore		goes directly to start

doRcl		GOTO	doRecall	goes directly to start

doDel		ST=0	sSAVE
		GOTO	restartMain

doOff		GOTO	turnOff		goto sleep, then goto start
**********************************************************************
* End of key subroutines
**********************************************************************
passKeys	C=RSTK
		?B=0	A
		GOYES	+
		B=B-1	A
		C=C+B	A
		C=C+B	A
		C=C+B	A
+		D0=C
		A=0	A
		A=DAT0	X
		C=C+A	A
		PC=C			branch to key routine
commonKey	GOSUB	DRAWLEV
		GOSUB	SWAPBUF
		?ST=0	sWAIT
		GOYES	+
		GOSUB	WAITNOKEY
+		GOSUB	LEVDONE?
		GOC	keepOnRocking	CC: level done, wait then go to next lev
		LC(5)	10000
		GOSUBL	DELAYC
		ST=0	sSAVE
		GOSUB	LEV+
		GOTO	restartMain
keepOnRocking	GOTO	MAINLP
doExit		GOSUB	WAITNOKEY	wait for user to release key
		GOSBVL	=AllowIntr	enable interrupts
		GOVLNG	=GETPTRLOOP
*********************************
* END OF MAIN LOOP
*********************************
**********************************************************************
* Kbd?
* Any keys pressed?
**********************************************************************
Kbd?		C=0	A
		LCHEX	1FF
keysub		OUT=C
		GOSBVL	=CINRTN
		?C#0	A
		RTNYES
		RTN
**********************************************************************
* doRecall
* Recall saved game
**********************************************************************
doRecall	C=R3			push level #
		D1=C
		D1+	LEV
		C=0	A
		C=DAT1	B
		R0=C
		GOSBVL	=PUSH#		does GETPTR
		GOSBVL	=SAVPTR
		GOSUB	GetptrEvalC
RPL
::
( Stack: 3:Levels 2:Buffer 1: #Level )
* ROMPTR 3E0 007	( SHALT )
 ' ID SOpar
 Sys@
 ITE
 ::		( we have a stored object )
 ( stack: 4:Levels 3:Buffer 2:#Level 1:Stored )
  ROTROT2DROP	( drop old buffer & #level )
  TOTEMPOB	( dereference object )
CODE
	A=DAT1	A
	AD1EX
	LC(5)	=DOLIST
	DAT1=C	A
	D1=D1+	5
	LC(5)	=DOBINT
	DAT1=C	A
	D1=A
	GOVLNG	=Loop
ENDCODE
* { #Level LibData }
  INCOMPDROP
ASSEMBLE
	CON(5)	=DOBINT
	CON(5)	(SAVED)+10
RPL
  OVER OSIZE
  #<>case
  ::
   2DROP
   $ "Bad SOpar!"
   DO$EXIT
  ;
ASSEMBLE
	CON(5)	=DOBINT
	CON(5)	34*64
RPL
  EXPAND
  SWAPTRUE
 ;
 TRUE	( there was no stored level, reload current )
 COLA
ASSEMBLE
		CON(5)	=DOCODE
		REL(5)	->end
		GOTO	start
**********************************************************************
* doStore
* Store current game
**********************************************************************
doStore		C=R3			push level #
		D1=C
		D1+	LEV
		C=0	A
		C=DAT1	B
		R0=C
		GOSBVL	=PUSH#		does GETPTR
		GOSBVL	=SAVPTR
		GOSUB	GetptrEvalC
RPL
::
( Stack: 3:Levels 2:Buffer 1:#Level )
* ROMPTR 3E0 007	( SHALT )
 2DUP
 TOTEMPSWAP
 TOTEMPOB
 ONE
ASSEMBLE
	CON(5)	=DOBINT
	CON(5)	(SAVED)/2
RPL
  SUB$
  TWO{}N
  DUP OSIZE
* { #Level LibData }
* Convert into a LibraryData
CODE
	GOSBVL	=POP#
	GOSBVL	=SAVPTR
	C=DAT1	A
	D1=C
	LC(5)	=DOEXT0
	DAT1=C	A
	D1=D1+	5
	A=A-CON	A,5		this was bugged!
	DAT1=A	A
	GOVLNG	=GETPTRLOOP
ENDCODE
  ' ID SOpar
  SysSTO
 TRUE
 COLA
ASSEMBLE
		CON(5)	=DOCODE
		REL(5)	->end
		GOTO	start
**********************************************************************
* turnOff
* Goto deep sleep...zzzz
**********************************************************************
turnOff		C=R3		push level #
		D1=C
		D1+	LEV
		C=0	A
		C=DAT1	B
		R0=C
		GOSBVL	=PUSH#
		GOSBVL	=SAVPTR
		GOSUB	GetptrEvalC
RPL
::
* ROMPTR 3E0 007	( SHALT )
 TurnOff
 TRUE
 COLA
ASSEMBLE
		CON(5)	=DOCODE
		REL(5)	->end
		GOTO	start
GetptrEvalC	C=RSTK
		A=C	A
		GOSBVL	=GETPTR
		PC=(A)
**********************************************************************
* LEV+
* Advance level
**********************************************************************
LEV+		C=R3
		D1=C
		D1+	LEV
		A=DAT1	B
		A=A+1	B
		LC(2)	MAXLEV
		?A<=C	B
		GOYES	+
		A=0	B
		A=A+1	B
+		DAT1=A	B
		RTN
**********************************************************************
* LEV-
* Decrease level
**********************************************************************
LEV-		C=R3
		D1=C
		D1+	LEV
		C=DAT1	B
		C=C-1	B
		?C#0	B
		GOYES	+
		LC(2)	MAXLEV
+		DAT1=C	B
		RTN
**********************************************************************
* WAITNOKEY
* Wait for user to release key
**********************************************************************
WAITNOKEY	GOSUB	Kbd?
		GOC	WAITNOKEY
		RTN
**********************************************************************
* TOGDelay
* Toggle fast mode
**********************************************************************
TOGDelay	?ST=1	sWAIT
		GOYES	+
		ST=1	sWAIT
		GOSUB	IO_OFF
		RTN
+		ST=0	sWAIT
		GOSUB	IO_ON
		RTN
**********************************************************************
* clrbusy
* Turn off busy ann
**********************************************************************
clrbusy		D1=(5)	=ANNCTRL
		C=DAT1	B
		CBIT=0	4
		DAT1=C	B
		RTN
**********************************************************************
* showbusy
* Turn on busy ann
**********************************************************************
showbusy	D1=(5)	=ANNCTRL
		C=DAT1	B
		CBIT=1	4
		DAT1=C	B
		RTN
**********************************************************************
* SWAPBUF
* Copy offscreen buffer to display
**********************************************************************
SWAPBUF		GOSBVL	=D0->Row1
		D1=A
		C=R1
		D0=C
		LC(5)	34*64
		GOVLNG	=MOVEDOWN
**********************************************************************
* KeyNum
* Return B.B = key held A.W = bitmap
**********************************************************************
KeyNum		GOSBVL	=Debounce	get bitmap to A.W
		B=0	A
		?A=0	W
		RTNYES
nextNib		?A#0	P
		GOYES	testBits
		B=B+CON	B,4
		ASR	W
		GONC	nextNib
* bet
testBits	B=B+1	B
		SB=0
		ASRB.F	P
		?SB=0
		GOYES	testBits
		RTN
**********************************************************************
* ThisKey?
* C.B pressed?
**********************************************************************
ThisKey?	RSTK=C
		GOSUB	KeyNum
		C=RSTK
		?B=C	B
		RTNYES
		RTN
**********************************************************************
* GO_UP
* Move cursor up one step
**********************************************************************
GO_UP		GOSUB	FINDMARK	D1->man
		D1-	MAXW
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	SPACE		' '
		?A#C	P
		GOYES	+
* ' ' MOVE + UP 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1+	MAXW
		DAT1=A	P
		RTN
+		LC(1)	GOAL		'.'
		?A#C	P
		GOYES	+
* . MOVE UP 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1+	MAXW
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
*  CHECK 2 UP
+		D1-	MAXW		D1->UP2
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	STONE		'$'
		?A=C	P
		RTNYES
		DAT1=C	P
		D1+	MAXW
		LC(1)	MAN		'@'
		DAT1=C	P
		D1+	MAXW
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
**********************************************************************
GO_DOWN		GOSUB	FINDMARK
		D1+	MAXW
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	SPACE		' '
		?A#C	P
		GOYES	+
* ' ' MOVE + DOWN 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1-	MAXW
		DAT1=A	P
		RTN
+		LC(1)	GOAL		'.'
		?A#C	P
		GOYES	+
* . MOVE DOWN 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1-	MAXW
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
*  CHECK 2 DOWN
+		D1+	MAXW		D1->DOWN2
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	STONE		'$'
		?A=C	P
		RTNYES
		DAT1=C	P
		D1-	MAXW
		LC(1)	MAN		'@'
		DAT1=C	P
		D1-	MAXW
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
**********************************************************************
GO_LEFT		GOSUB	FINDMARK
		D1-	1
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	SPACE		' '
		?A#C	P
		GOYES	+
* ' ' MOVE + LEFT 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1=D1+	1
		DAT1=A	P
		RTN
+		LC(1)	GOAL		'.'
		?A#C	P
		GOYES	+
* . MOVE LEFT 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1=D1+	1
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
*  CHECK 2 LEFT
+		D1=D1-	1		D1->LEFT2
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	STONE		'$'
		?A=C	P
		RTNYES
		DAT1=C	P
		D1=D1+	1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1=D1+	1
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
**********************************************************************
GO_RIGHT	GOSUB	FINDMARK
		D1=D1+	1
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	SPACE		' '
		?A#C	P
		GOYES	+
* ' ' MOVE + RIGHT 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1=D1-	1
		DAT1=A	P
		RTN
+		LC(1)	GOAL		'.'
		?A#C	P
		GOYES	+
* . MOVE RIGHT 1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1=D1-	1
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
*  CHECK 2 RIGHT
+		D1=D1+	1		D1->RT2
		A=DAT1	B
		LC(1)	WALL		'#'
		?A=C	P
		RTNYES
		LC(1)	STONE		'$'
		?A=C	P
		RTNYES
		DAT1=C	P
		D1=D1-	1
		LC(1)	MAN		'@'
		DAT1=C	P
		D1=D1-	1
		LC(1)	SPACE		' '
		DAT1=C	P
		RTN
**********************************************************************
* DRAWLEV: Draw level from rambuffer into offscreen buffer.
* If level contains unknown characters we error out
* LEV.RAM = (in) current level from ram
* R1.A = (out) offscreen buffer
**********************************************************************
DRAWLEV		C=R3
		D0=C
		D0+	LEV.RAM		D0-> current lev (ram)
		C=R1
		D1=C			D1-> offscreen buffer
		C=0	A
		LC(2)	MAXH
		D=C	A
--		C=0	A
		LC(2)	MAXW
		B=C	A
-		A=DAT0	B
		D0=D0+	1
		LC(1)	WALL		'#'
		?C#A	P
		GOYES	+
		LC(4)	#A5A5
		GONC	++
+		LC(1)	MAN		'@'
		?C#A	P
		GOYES	+
		LC(4)	#0272
		GONC	++
+		LC(1)	STONE		'$'
		?C=A	P
		GOYES	drawBox
		LC(1)	FILLED		'*'
		?C#A	P
		GOYES	+
drawBox		LC(4)	#0757
		GOTO	++
+		LC(1)	GOAL		'.'
		?C#A	P
		GOYES	+
		LC(4)	#0020
		GONC	++
+		C=0	A
++		DAT1=C	P
		D1+	34
		P=	1
		DAT1=C	P
		D1+	34
		P=	2
		DAT1=C	P
		D1+	34
		P=	3
		DAT1=C	P
		P=	0
		LC(5)	34*3-1
		AD1EX
		A=A-C	A
		AD1EX
		B=B-1	A
		?B=0	A
		GOYES	+
		GOTO	-
+		D=D-1	A
		?D=0	A
		GOYES	DOPOINTS
		LC(5)	34*3+(34-20)
		AD1EX
		A=A+C	A
		AD1EX
		GOTO	--
* Fill in points
DOPOINTS	C=R2
		D0=C			->LEV.ROM
		C=R1
		D1=C			->offscreen
		C=0	A
		LC(2)	MAXH
		D=C	A
--		C=0	A
		LC(2)	MAXW
		B=C	A
-		A=DAT0	B
		D0=D0+	1
		LC(1)	GOAL		'.'
		?C=A	P
		GOYES	point
		LC(1)	FILLED		'*'
		?C#A	P
		GOYES	+
point		LC(4)	#0020
		GOTO	++
+		C=0	A
++		A=DAT1	P
		C=C!A	P
		DAT1=C	P
		D1+	34
		P=	1
		A=DAT1	P
		C=C!A	P
		DAT1=C	P
		D1+	34
		P=	2
		A=DAT1	P
		C=C!A	P
		DAT1=C	P
		D1+	34
		P=	3
		A=DAT1	P
		C=C!A	P
		DAT1=C	P
		P=	0
		LC(5)	34*3-1
		AD1EX
		A=A-C	A
		AD1EX
		B=B-1	A
		?B=0	A
		GOYES	+
		GOTO	-
+		D=D-1	A
		?D=0	A
		RTNYES
		LC(5)	34*3+(34-20)
		AD1EX
		A=A+C	A
		AD1EX
		GOTO	--
**********************************************************************
* FINDMARK
* Find man in level
* Return D1->position of man
**********************************************************************
FINDMARK	C=R3
		D1=C
		D1+	LEV.RAM
		LC(1)	MAN	'@'
		A=C	B
-		C=DAT1	B
		?C=A	P
		RTNYES
		D1=D1+	1
		GONC	-
**********************************************************************
* WIPEOUT
* Zero buffer pointed to by D1
* C.A = nibbles
**********************************************************************
WIPEOUT		A=0	W
		C=C-1	A
		GOC	wRtn
		P=C	0
		CSR	A
		C=C-1	A
		GOC	wP
-		DAT1=A	W
		D1=D1+	16
		C=C-1	B
		GONC	-
		C=C-1	XS
		GONC	-
		C=C+1	X
		C=C-1	A
		GONC	-
wP		DAT1=A	WP
		CD1EX
		C+P+1
		D1=C
wRtn		P=	0
		RTNCC
**********************************************************************
* InitLev
* Copy lev into ram
* Zero offscreen buffer
**********************************************************************
InitLev		C=R1
		D1=C			->offscreen
		LC(5)	34*64
		GOSUB	WIPEOUT		zero offscreen buffer
		C=R3
		D0=C
		D0+	LEV		->level#
		C=0	A
		C=DAT0	B
		C=C-1	A
		LA(5)	LEVSZ
		GOSBVL	=MUL#
		A=R0			->levels
		C=B	A
		C=C+A	A		C.A->level in rom
		R2=C			save in R2 (permanently)
		?ST=1	sSAVE		level read from stack?
		RTNYES			yes, do not copy lev from rom
		D0=C			->level in rom
		C=R3
		D1=C			->buffer
		D1+	LEV.RAM		->level in ram
		LC(5)	(LEVSZ)-1
		B=C	A
		LC(1)	FILLED	'*'	if we read a * we put a $
		A=C	B
copyLp		C=DAT0	B
		?C=A	P
		GOYES	dollar
		DAT1=C	P		else we just put what we read
		GONC	+
dollar		LC(1)	STONE	'$'
		DAT1=C	P
+		D0=D0+	1
		D1=D1+	1
		B=B-1	A
		GONC	copyLp
		RTN
**********************************************************************
* LEVDONE?
* CS=NOT DONE
* CC=DONE
**********************************************************************
LEVDONE?	C=R3
		D1=C
		D1+	LEV.RAM		->LEV.RAM
		C=R2
		D0=C			->LEV.ROM
		LC(5)	(LEVSZ)-1
		B=C	A
-		LC(1)	GOAL		'.'
		A=DAT0	B
		?C=A	P
		GOYES	occupied?
		LC(1)	FILLED		'*'
		?C#A	P
		GOYES	+
* here we have either . or *
occupied?	A=DAT1	B
		LC(1)	STONE		'$' box here?
		?A#C	P		if not, return, CS
		RTNYES
+		D1=D1+	1
		D0=D0+	1
		B=B-1	A
		GONC	-
		RTNCC			level done, CC
**********************************************************************
* INITVARS
* Pop vars from stack
* 4: Levels		4:
* 3: Buffer	-or-	3: Levels
* 2: #Level		2: Buffer
* 1: TRUE		1: FALSE
**********************************************************************
InitVars	ST=0	sSAVE
		GOSBVL	=popflag
		GONC	popBuffer
		ST=1	sSAVE
		GOSBVL	=POP#
		R2=A			save level here a while
* stack: ( Levels Buffer )
popBuffer	C=DAT1	A
		C=C+CON	A,10		10 for the prolog & size fields
		A=C	A
		R3=C			->buffer
		LC(5)	OFFSCR
		C=C+A	A
		R1=C			R1.A=offscreen buffer
		AD1EX			save D1, D1->buffer
		D1+	LEV
		C=R2
		DAT1=C	B		write level
		D1=A			restore D1
		GOSBVL	=SAVPTR
		D1=D1+	5
		C=DAT1	A		read levels
		C=C+CON	A,10
		R0=C			->Levels
		D1=(5)	=ANNCTRL
		ST=1	sWAIT		assume slow movement
		C=DAT1	B
		?CBIT=0	5		is it slow?
		GOYES	moveSlow	yes
		ST=0	sWAIT		no, fast movement
moveSlow	?ST=1	sSAVE		#level from stack?
		RTNYES			yes, return
		C=R3			no, start at 1
		D0=C
		D0+	LEV
		C=0	A
		C=C+1	A
		DAT0=C	B
		RTN
**********************************************************************
* IO_OFF
* Turn off io ann
**********************************************************************
IO_OFF		D1=(5)	=ANNCTRL
		C=DAT1	B
		CBIT=0	5
		DAT1=C	B
		RTN
**********************************************************************
* IO_ON
* Turn on io ann
**********************************************************************
IO_ON		D1=(5)	=ANNCTRL
		C=DAT1	B
		CBIT=1	5
		DAT1=C	B
		RTN
**********************************************************************
* DELAYC
* Wait C.A ticks
**********************************************************************
DELAYC		?ST=1	sWAIT
		RTNYES
		D1=(5)	=TIMER2
		A=DAT1	A
		A=A-C	A
DLLP		C=DAT1	A
		?C>A	A
		GOYES	DLLP
		RTN
**********************************************************************
* DrawLev#
* Draw current level #
* Font format: 19*64,
* so we have 19/4 = 6 nibbles wide because we round up to nearest even
* nibble, and 64 pixels high
* Each digit is 6*64 nibbles = 384d = 180h
* If number is less then 10, we only draw one digit,
* not the leading zero, and we also center the digit on the drawing area
**********************************************************************
DrawLev#	A=R1			A.A->offscreen buffer
		C=0	A
		LC(2)	21
		C=C+A	A
		D1=C			D1->drawing area
		C=R3
		D0=C
		D0+	LEV		->level#
		C=0	W
		LC(2)	9*16+16-2
		CSRC			C.S=counter=16-2
		D=C	B		D.B=9
		C=DAT0	B
		A=C	A		A.B=level#
* Convert A.B into decimal
* result will be in B.B
* Code taken from UNCOERCE
		LC(2)	25
		B=0	W
-		B=B+B	W
		SETHEX
		A=A+A	A
		SETDEC
		GONC	+
		B=B+1	W
+		C=C-1	B
		GONC	-
		SETHEX
		C=B	B		C.B=level in decimal
		?C>D	B		digit>9?
		GOYES	two
		C=C+1	S		counter=16-1
		D1=D1+	4		digit<=9, so advance a little on drawing area
* C.A=number, C.S=counter, D1->drawing area
two		A=C	A		A.A=number
		GOSUBL	passFont
		NIBHEX	00000000F00008F1000CF3000CF7000EFF000EFF000F3C100F1C100F08308F083087003087007087
		NIBHEX	0070C30070C30070C30060C30060C10060C10060E10060E10060E10060E10060E10060E00060E000
		NIBHEX	60E00060E00060F00060F00060F00060F00060F00060F00060F00060F00060F00070F00070F00070
		NIBHEX	F00070E00070E00070E00070E00870E00870E00870E00830E00C30C10C30C10C30C10E10830E1083
		NIBHEX	0F10870F000F8F000EF7000EF7000CF30000F100004000000000000000000000
		NIBHEX	00000000000000000000600000F00000F00008F00008F10008F10008F10008F10008F10000E10000
		NIBHEX	E10000E10000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E0
		NIBHEX	0000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E000
		NIBHEX	00E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000E00000
		NIBHEX	E10000F30008F30008F30008F30000F30000F10000E000000000000000000000
		NIBHEX	000000000000000700008F0000CF1000EF1000EF3000FF3000F37008F17008F0700CF0700C70700E
		NIBHEX	30700E30700E10700F18700F08708F0870870830830C30830C30830C30810E30810E10000E10000F
		NIBHEX	10000F00000F0000870000870000870000C30000C30000E10000E10000E00000F000007020087060
		NIBHEX	0870600C30600C30600C10600E10700E00700E0030070030070030030C30838F3083FF30CFFF30CF
		NIBHEX	FF30CFFF30CFFF308FF7008F3000870000000000000000000000000000000000
		NIBHEX	00000000000000000000800000E30000F70008FF0008FF100CFF100EFF100EF8300F70300F30308F
		NIBHEX	10308F0830C70830C30830C10C30C10C30C00E30C00E30400F10000F10008F00008F0000C70000E7
		NIBHEX	0000E30000F10008F1000CF0000CF0000CF1000CF3000CF7000CF7000CFF0008FF00000E10000C10
		NIBHEX	000C10000810000810000830000830000C30000C30000E30000E10000F10408F10C0EF10CDFF00CF
		NIBHEX	FF00CFF7008FF3008FF1000FF0000E70000C0000000000000000000000000000
		NIBHEX	00000000000000000000000000810000830000C30000C70000E70000E70000F70000F70008F70008
		NIBHEX	77000C37000C37000C17000E97000E87000E87000F8300078300878300878300838300C38300C383
		NIBHEX	00C18300E18300E18300E08300F08300708300708300708300FFF300FFFF00FFFF10FFFF10FFFF10
		NIBHEX	FFFF10FFCB10F0810000810000810000810000810000810000C10000C10000C10000C10000C30000
		NIBHEX	C70000E70000FF0000FF0000FF0000F70000E70000C100000000000000000000
		NIBHEX	000000000000000000000000000F1008FF300FFF300FFF308FFF308FFF308FF9308F30308F003087
		NIBHEX	00308700308300308300308300308300308300308300308300308300008300008300008300008300
		NIBHEX	008300008300008F70008FF1008FF3008FF3008FF7008FFF008F1F00870E10030C10000830000830
		NIBHEX	000830000030000030000030000030000030000830000830000C30000E10000E10000F10008F0040
		NIBHEX	EF00C8F700CFF300CFF3008FF1008FF0000F30000C0000000000000000000000
		NIBHEX	00000000000000000000C10000E30000F70008FF0008FF000CFF100CFD100E78100E30100E10000F
		NIBHEX	10000F00008F0000870000870000830000C30000C30000C30000C10000C10000E10000E10000E8F0
		NIBHEX	00ECF100EFF300EFF700EFF700FF3E00FF0C00F70C10F30810F10810F00810F00830700830700830
		NIBHEX	700830700830700830700830700830700830600C30600C30600C30600C10E00E10E00E10C00F10C1
		NIBHEX	0F00C18F0083CF0087E7000FF3000EF3000EF000087000000000000000000000
		NIBHEX	000000000000000000000000000E0000CF1008FF300CFF300FFF308FFF308FFF308FFD308F1C3083
		NIBHEX	0C30810E10810E10810E10810E00810F00810F008107008107008187008187008183008183000083
		NIBHEX	0000C30000C10000C10000C10000E10000E00000E00000E00000F000007000007000007000007000
		NIBHEX	0830000830000830000830000C10000C10000C10000C10000E00000E00000E00000E00000700000F
		NIBHEX	0000CF1000CF3000CF3000CF3000CF3000CF1000830000000000000000000000
		NIBHEX	00000000000000F00008F1400CF3600EF3600FF7700F1F708F0F70870E70C70E70C30E30C30E10C1
		NIBHEX	0F10C10F00C18F00C18700C1C700C3C300C3E30083E10083F10087F1000FF0000FF0000E70000E70
		NIBHEX	000E30000E70000E70000FF0000FF1008FE30087C300C78700C30F00C30E00E30E00E10C10E10C10
		NIBHEX	E10810F00810F00810F00810F00810700810700C10700C10700C10700C10700E10300E10300F0030
		NIBHEX	0F00308F0070C70070E700E8F300EFF300CFF100CFF0008F70000E3000000000
		NIBHEX	00000000000000700008F1000CF3000EF7000EF7000FFF000F3C100F18108F08108F083087083087
		NIBHEX	0030C70030C30070C30070C30070C30070C10070C10070C10070C10070C10070C10070C10070C100
		NIBHEX	70C10070C10870830870830C70870F700F9F700EFF700EFF300CFF3008FF3000EB30000830000830
		NIBHEX	000C30000C10000C10000C10000E10000E10000E00000F00000F00008F0000870000C700C0C300C1
		NIBHEX	F300CFF3008FF1000FF1000FF0000E7000083000000000000000000000000000
passFont	C=RSTK			ease jazz disassembly
		RSTK=C
		P=C	15		Save number in high A[W]
saveNumLp	ASRC
		P=P+1
		GONC	saveNumLp
		A=0	A
		ASLC			A[A] = digit
		C=RSTK			C[A] = ->font
		RSTK=C			save ->font here
		ASL	A		*10h
		B=A	A
		B=B+B	A		*20h
		ASL	A		*100h
		A=A+B	A		*120h
		A=A+B	A		*140h
		A=A+B	A		*160h
		A=A+B	A		*180h
		C=C+A	A
		D0=C			->font+180h*digit
		LC(2)	64-1
		D=C	B		D.B=rows-1
drawDigitLp	C=DAT0	6		read one line
		DAT1=C	6		write one line
		D0=D0+	6
		D1+	34
		D=D-1	B
		GONC	drawDigitLp
		C=C+1	S
		GONC	secondDigit	carry? if C.S=0 then we're done
		C=RSTK			need to pop font address
		RTN
secondDigit	LC(5)	34*64-6
		AD1EX
		A=A-C	A
		AD1EX
*		D1=D1-C			point D1 to next drawing area
* draw the second digit
		A=0	A
		ASLC			A[A] = digit
		C=RSTK			C[A] = ->font, no need to save it
		ASL	A		*10h
		B=A	A
		B=B+B	A		*20h
		ASL	A		*100h
		A=A+B	A		*120h
		A=A+B	A		*140h
		A=A+B	A		*160h
		A=A+B	A		*180h
		C=C+A	A
		D0=C			->font+180h*digit
		LC(2)	64-1
		D=C	B		D.B=rows-1
drawDigit2Lp	C=DAT0	6		read one line
		DAT1=C	6		write one line
		D0=D0+	6
		D1+	34
		D=D-1	B
		GONC	drawDigit2Lp
		RTN
->end
RPL
 2DROP	( drop Levels & Buffer )
;
